
local tablet = AceLibrary("Tablet-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local path, query, target = {}
local types = {
	["table"] = true,
	["function"] = true,
	["string"] = true,
	["boolean"] = true,
	["number"] = true,
	["userdata"] = true,
}

tekProber = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0", "AceModuleCore-2.0")
tekProber:RegisterDB("tekProberDB", "tekProberDBPC")
tekProber.hasNoText = true
tekProber.clickableTooltip = true
tekProber.hasIcon  = "Interface\\Icons\\Spell_ChargeNegative"
tekProber.OnMenuRequest = {
	type = "group",
	args   = {
		["set"] = {
			type = "text",
			name = "Set table",
			desc = "Set a script to retrieve the table you wish to view.",
			handler = tekProber,
			get = function() return query end,
			set = "SetQuery",
			usage = "<Any string>",
		},
	},
}
tekProber:RegisterChatCommand({"/probe"}, tekProber.OnMenuRequest)


function tekProber:SetQuery(q)
	query = q or query
	RunScript("function tekProber:GetTarget() return " .. query .. " end")
	repeat table.remove(path) until not next(path)
	self:OnClick()
end


function tekProber:OnClick()
	target = self.GetTarget and self:GetTarget()
	self:Update()
end


function tekProber:OnTooltipUpdate()
	if type(target) ~= "table" then
		local cat = tablet:AddCategory("text", "No table to display", "showWithoutChildren", true)
		return
	end

	local n = table.getn(path)
	if next(path) then t = self:DelvePath()
	else t = target end

	if not t then return end

	local cat = tablet:AddCategory("text", query, "showWithoutChildren", true)
	for i,v in ipairs(path) do cat:AddLine("text", v, "func", self.PathBack, "arg1", self) end

	for i,v in pairs(types) do types[i] = tablet:AddCategory("columns", 2, "text", i) end
	for i,v in pairs(t) do
		local vt = type(v)
		assert(types[vt], "Type category does not exist: "..vt)
		types[vt]:AddLine("text", tostring(i),
			"text2", vt == "table" and "" or tostring(v),
			vt == "table" and "func", vt == "table" and self.AddToPath,
			vt == "table" and "arg1", vt == "table" and self,
			vt == "table" and "arg2", vt == "table" and i)
	end
end


function tekProber:DelvePath(t, depth)
	local n = table.getn(path)
	depth = depth or 1
	t = t or target
	assert(t, "No target to delve")

	if n == depth then return t[path[depth]]
	else return self:DelvePath(t[path[depth]], depth + 1) end
end


function tekProber:AddToPath(val)
	table.insert(path, val)
	self:UpdateTooltip()
end


function tekProber:PathBack()
	table.remove(path)
	self:UpdateTooltip()
end
